@@ -13,8 +13,8 @@ int create_server_socket(const sockaddr_in6 &addr, SocketType socket_type);
 
 class AcceptorProto;
 
-sockaddr_in6 CreateAnyAddress(int port);
-sockaddr_in6 ExtractAddressFromAcceptorProto(const AcceptorProto &proto);
+sockaddr_in6 create_any_address(int port);
+sockaddr_in6 extract_address_from_acceptor_proto(const AcceptorProto &proto);
 
 // A thread that accepts new connections on a given socket,
 // and hands them off to the server pool.
