@@ -242,7 +242,7 @@ bool parse_stream(const ConfigLine &line, Config *config)
 		stream.prebuffering_bytes = atoi(prebuffer_it->second.c_str());
 	}
 
-	// Parse encoding.
+	// Parse output encoding.
 	map<string, string>::const_iterator encoding_parm_it = line.parameters.find("encoding");
 	if (encoding_parm_it == line.parameters.end() ||
 	    encoding_parm_it->second == "raw") {
@@ -254,6 +254,18 @@ bool parse_stream(const ConfigLine &line, Config *config)
 		return false;
 	}
 
+	// Parse input encoding.
+	map<string, string>::const_iterator src_encoding_parm_it = line.parameters.find("src_encoding");
+	if (src_encoding_parm_it == line.parameters.end() ||
+	    src_encoding_parm_it->second == "metacube") {
+		stream.src_encoding = StreamConfig::STREAM_ENCODING_METACUBE;
+	} else if (src_encoding_parm_it->second == "raw") {
+		stream.src_encoding = StreamConfig::STREAM_ENCODING_RAW;
+	} else {
+		log(ERROR, "Parameter 'src_encoding' must be either 'raw' or 'metacube' (default)");
+		return false;
+	}
+
 	// Parse the pacing rate, converting from kilobits to bytes as needed.
 	map<string, string>::const_iterator pacing_rate_it = line.parameters.find("pacing_rate_kbit");
 	if (pacing_rate_it == line.parameters.end()) {
