@@ -15,7 +15,9 @@ struct StreamConfig {
 	size_t backlog_size;
 	size_t prebuffering_bytes;
 	uint32_t pacing_rate;  // In bytes per second. Default is ~0U (no limit).
-	enum { STREAM_ENCODING_RAW = 0, STREAM_ENCODING_METACUBE } encoding;
+	enum Encoding { STREAM_ENCODING_RAW = 0, STREAM_ENCODING_METACUBE };
+	Encoding encoding;
+	Encoding src_encoding;
 };
 
 struct UDPStreamConfig {
