@@ -7,6 +7,7 @@
 #include <vector>
 
 #include "input.h"
+#include "metacube2.h"
 
 class InputProto;
 
@@ -44,6 +45,8 @@ private:
 	// and outputs a warning.
 	void drop_pending_data(size_t num_bytes);
 
+	void process_metacube_metadata_block(const metacube2_block_header &hdr, const char *payload, uint32_t payload_size);
+
 	enum State {
 		NOT_CONNECTED,
 		SENDING_REQUEST,
