@@ -39,6 +39,8 @@ ServerPool *servers = NULL;
 volatile bool hupped = false;
 volatile bool stopped = false;
 
+typedef pair<string, Input::Encoding> InputKey;
+
 namespace {
 
 struct OrderByConnectionTime {
@@ -70,7 +72,7 @@ void do_nothing(int signum)
 
 CubemapStateProto collect_state(const timespec &serialize_start,
                                 const vector<Acceptor *> acceptors,
-                                const multimap<string, InputWithRefcount> inputs,
+                                const multimap<InputKey, InputWithRefcount> inputs,
                                 ServerPool *servers)
 {
 	CubemapStateProto state = servers->serialize();  // Fills streams() and clients().
@@ -81,7 +83,7 @@ CubemapStateProto collect_state(const timespec &serialize_start,
 		state.add_acceptors()->MergeFrom(acceptors[i]->serialize());
 	}
 
-	for (multimap<string, InputWithRefcount>::const_iterator input_it = inputs.begin();
+	for (multimap<InputKey, InputWithRefcount>::const_iterator input_it = inputs.begin();
 	     input_it != inputs.end();
 	     ++input_it) {
 		state.add_inputs()->MergeFrom(input_it->second.input->serialize());
@@ -124,44 +126,45 @@ vector<Acceptor *> create_acceptors(
 	return acceptors;
 }
 
-void create_config_input(const string &src, multimap<string, InputWithRefcount> *inputs)
+void create_config_input(const string &src, Input::Encoding encoding, multimap<InputKey, InputWithRefcount> *inputs)
 {
 	if (src.empty()) {
 		return;
 	}
-	if (inputs->count(src) != 0) {
+	InputKey key(src, encoding);
+	if (inputs->count(key) != 0) {
 		return;
 	}
 
 	InputWithRefcount iwr;
-	iwr.input = create_input(src);
+	iwr.input = create_input(src, encoding);
 	if (iwr.input == NULL) {
-		log(ERROR, "did not understand URL '%s', clients will not get any data.",
+		log(ERROR, "did not understand URL '%s' or source encoding was invalid, clients will not get any data.",
 			src.c_str());
 		return;
 	}
 	iwr.refcount = 0;
-	inputs->insert(make_pair(src, iwr));
+	inputs->insert(make_pair(key, iwr));
 }
 
 // Find all streams in the configuration file, and create inputs for them.
-void create_config_inputs(const Config &config, multimap<string, InputWithRefcount> *inputs)
+void create_config_inputs(const Config &config, multimap<InputKey, InputWithRefcount> *inputs)
 {
 	for (unsigned i = 0; i < config.streams.size(); ++i) {
 		const StreamConfig &stream_config = config.streams[i];
 		if (stream_config.src != "delete") {
-			create_config_input(stream_config.src, inputs);
+			create_config_input(stream_config.src, Input::Encoding(stream_config.src_encoding), inputs);
 		}
 	}
 	for (unsigned i = 0; i < config.udpstreams.size(); ++i) {
 		const UDPStreamConfig &udpstream_config = config.udpstreams[i];
-		create_config_input(udpstream_config.src, inputs);
+		create_config_input(udpstream_config.src, Input::INPUT_ENCODING_RAW, inputs);
 	}
 }
 
 void create_streams(const Config &config,
                     const set<string> &deserialized_urls,
-                    multimap<string, InputWithRefcount> *inputs)
+                    multimap<InputKey, InputWithRefcount> *inputs)
 {
 	// HTTP streams.
 	set<string> expecting_urls = deserialized_urls;
@@ -181,7 +184,8 @@ void create_streams(const Config &config,
 			stream_index = servers->add_stream(stream_config.url,
 			                                   stream_config.backlog_size,
 			                                   stream_config.prebuffering_bytes,
-			                                   Stream::Encoding(stream_config.encoding));
+			                                   Stream::Encoding(stream_config.encoding),
+			                                   Stream::Encoding(stream_config.src_encoding));
 		} else {
 			stream_index = servers->lookup_stream_by_url(stream_config.url);
 			assert(stream_index != -1);
@@ -189,13 +193,16 @@ void create_streams(const Config &config,
 			servers->set_prebuffering_bytes(stream_index, stream_config.prebuffering_bytes);
 			servers->set_encoding(stream_index,
 			                      Stream::Encoding(stream_config.encoding));
+			servers->set_src_encoding(stream_index,
+			                          Stream::Encoding(stream_config.src_encoding));
 		}
 
 		servers->set_pacing_rate(stream_index, stream_config.pacing_rate);
 
 		string src = stream_config.src;
+		Input::Encoding src_encoding = Input::Encoding(stream_config.src_encoding);
 		if (!src.empty()) {
-			multimap<string, InputWithRefcount>::iterator input_it = inputs->find(src);
+			multimap<InputKey, InputWithRefcount>::iterator input_it = inputs->find(make_pair(src, src_encoding));
 			if (input_it != inputs->end()) {
 				input_it->second.input->add_destination(stream_index);
 				++input_it->second.refcount;
@@ -225,7 +232,7 @@ void create_streams(const Config &config,
 
 		string src = udpstream_config.src;
 		if (!src.empty()) {
-			multimap<string, InputWithRefcount>::iterator input_it = inputs->find(src);
+			multimap<InputKey, InputWithRefcount>::iterator input_it = inputs->find(make_pair(src, Input::INPUT_ENCODING_RAW));
 			assert(input_it != inputs->end());
 			input_it->second.input->add_destination(stream_index);
 			++input_it->second.refcount;
@@ -400,7 +407,7 @@ start:
 	timespec serialize_start;
 	set<string> deserialized_urls;
 	map<sockaddr_in6, Acceptor *, Sockaddr6Compare> deserialized_acceptors;
-	multimap<string, InputWithRefcount> inputs;  // multimap due to older versions without deduplication.
+	multimap<InputKey, InputWithRefcount> inputs;  // multimap due to older versions without deduplication.
 	if (state_fd != -1) {
 		log(INFO, "Deserializing state from previous process...");
 		string serialized;
@@ -445,7 +452,12 @@ start:
 			InputWithRefcount iwr;
 			iwr.input = create_input(serialized_input);
 			iwr.refcount = 0;
-			inputs.insert(make_pair(serialized_input.url(), iwr));
+
+			Input::Encoding src_encoding = serialized_input.is_metacube_encoded() ?
+				Input::INPUT_ENCODING_METACUBE :
+				Input::INPUT_ENCODING_RAW;
+			InputKey key(serialized_input.url(), src_encoding);
+			inputs.insert(make_pair(key, iwr));
 		} 
 
 		// Deserialize the acceptors.
@@ -503,11 +515,17 @@ start:
 	servers->run();
 
 	// Now delete all inputs that are longer in use, and start the others.
-	for (multimap<string, InputWithRefcount>::iterator input_it = inputs.begin();
+	for (multimap<InputKey, InputWithRefcount>::iterator input_it = inputs.begin();
 	     input_it != inputs.end(); ) {
 		if (input_it->second.refcount == 0) {
-			log(WARNING, "Input '%s' no longer in use, closing.",
-			    input_it->first.c_str());
+			if (input_it->first.second == Input::INPUT_ENCODING_RAW) {
+				log(WARNING, "Raw input '%s' no longer in use, closing.",
+				    input_it->first.first.c_str());
+			} else {
+				assert(input_it->first.second == Input::INPUT_ENCODING_METACUBE);
+				log(WARNING, "Metacube input '%s' no longer in use, closing.",
+				    input_it->first.first.c_str());
+			}
 			input_it->second.input->close_socket();
 			delete input_it->second.input;
 			inputs.erase(input_it++);
@@ -527,7 +545,7 @@ start:
 	InputStatsThread *input_stats_thread = NULL;
 	if (!config.input_stats_file.empty()) {
 		vector<Input*> inputs_no_refcount;
-		for (multimap<string, InputWithRefcount>::iterator input_it = inputs.begin();
+		for (multimap<InputKey, InputWithRefcount>::iterator input_it = inputs.begin();
 		     input_it != inputs.end(); ++input_it) {
 			inputs_no_refcount.push_back(input_it->second.input);
 		}
@@ -575,7 +593,7 @@ start:
 	for (size_t i = 0; i < acceptors.size(); ++i) {
 		acceptors[i]->stop();
 	}
-	for (multimap<string, InputWithRefcount>::iterator input_it = inputs.begin();
+	for (multimap<InputKey, InputWithRefcount>::iterator input_it = inputs.begin();
 	     input_it != inputs.end();
 	     ++input_it) {
 		input_it->second.input->stop();
