@@ -653,7 +653,7 @@ int Server::parse_request(Client *client)
 
 	string url = request_tokens[1];
 	client->url = url;
-	if (url.find("?backlog") == url.size() - 8) {
+	if (url.size() > 8 && url.find("?backlog") == url.size() - 8) {
 		client->stream_pos = -2;
 		url = url.substr(0, url.size() - 8);
 	} else {
