@@ -48,19 +48,20 @@ public:
 	// and the order between them are undefined.
 	// XXX: header should ideally be ordered with respect to data.
 	void add_client_deferred(int sock);
-	void add_data_deferred(int stream_index, const char *data, size_t bytes, StreamStartSuitability suitable_for_stream_start);
+	void add_data_deferred(int stream_index, const char *data, size_t bytes, uint16_t metacube_flags);
 
 	// These should not be called while running, since that would violate
 	// threading assumptions (ie., that epoll is only called from one thread
 	// at the same time).
 	CubemapStateProto serialize();
 	void add_client_from_serialized(const ClientProto &client);
-	int add_stream(const std::string &url, size_t bytes_received, size_t prebuffering_bytes, Stream::Encoding encoding);
+	int add_stream(const std::string &url, size_t bytes_received, size_t prebuffering_bytes, Stream::Encoding encoding, Stream::Encoding src_encoding);
 	int add_stream_from_serialized(const StreamProto &stream, int data_fd);
 	int lookup_stream_by_url(const std::string &url) const;
 	void set_backlog_size(int stream_index, size_t new_size);
 	void set_prebuffering_bytes(int stream_index, size_t new_amount);
 	void set_encoding(int stream_index, Stream::Encoding encoding);
+	void set_src_encoding(int stream_index, Stream::Encoding encoding);
 	void add_gen204(const std::string &url, const std::string &allow_origin);
 
 private:
