@@ -73,14 +73,14 @@ int ServerPool::lookup_stream_by_url(const string &url) const
 	return servers[0].lookup_stream_by_url(url);
 }
 
-int ServerPool::add_stream(const string &url, size_t backlog_size, size_t prebuffering_bytes, Stream::Encoding encoding)
+int ServerPool::add_stream(const string &url, size_t backlog_size, size_t prebuffering_bytes, Stream::Encoding encoding, Stream::Encoding src_encoding)
 {
 	// Adding more HTTP streams after UDP streams would cause the UDP stream
 	// indices to move around, which is obviously not good.
 	assert(udp_streams.empty());
 
 	for (int i = 0; i < num_servers; ++i) {
-		int stream_index = servers[i].add_stream(url, backlog_size, prebuffering_bytes, encoding);
+		int stream_index = servers[i].add_stream(url, backlog_size, prebuffering_bytes, encoding, src_encoding);
 		assert(stream_index == num_http_streams);
 	}
 	return num_http_streams++;
@@ -147,7 +147,7 @@ void ServerPool::set_header(int stream_index, const string &http_header, const s
 	}
 }
 
-void ServerPool::add_data(int stream_index, const char *data, size_t bytes, StreamStartSuitability suitable_for_stream_start)
+void ServerPool::add_data(int stream_index, const char *data, size_t bytes, uint16_t metacube_flags)
 {
 	assert(stream_index >= 0 && stream_index < ssize_t(num_http_streams + udp_streams.size()));
 
@@ -159,7 +159,7 @@ void ServerPool::add_data(int stream_index, const char *data, size_t bytes, Stre
 
 	// HTTP stream.
 	for (int i = 0; i < num_servers; ++i) {
-		servers[i].add_data_deferred(stream_index, data, bytes, suitable_for_stream_start);
+		servers[i].add_data_deferred(stream_index, data, bytes, metacube_flags);
 	}
 }
 
@@ -221,3 +221,10 @@ void ServerPool::set_encoding(int stream_index, Stream::Encoding encoding)
 		servers[i].set_encoding(stream_index, encoding);
 	}	
 }
+
+void ServerPool::set_src_encoding(int stream_index, Stream::Encoding encoding)
+{
+	for (int i = 0; i < num_servers; ++i) {
+		servers[i].set_src_encoding(stream_index, encoding);
+	}
+}
