@@ -29,7 +29,7 @@ public:
 	void add_client_from_serialized(const ClientProto &client);
 
 	// Adds the given stream to all the servers. Returns the stream index.
-	int add_stream(const std::string &url, size_t backlog_size, size_t prebuffering_bytes, Stream::Encoding encoding);
+	int add_stream(const std::string &url, size_t backlog_size, size_t prebuffering_bytes, Stream::Encoding encoding, Stream::Encoding src_encoding);
 	int add_stream_from_serialized(const StreamProto &stream, const std::vector<int> &data_fds);
 	void delete_stream(const std::string &url);
 	int add_udpstream(const sockaddr_in6 &dst, int pacing_rate, int ttl, int multicast_iface_index);
@@ -41,7 +41,7 @@ public:
 	void set_header(int stream_index,
 	                const std::string &http_header,
 	                const std::string &stream_header);
-	void add_data(int stream_index, const char *data, size_t bytes, StreamStartSuitability suitable_for_stream_start);
+	void add_data(int stream_index, const char *data, size_t bytes, uint16_t metacube_flags);
 
 	// Sets the max pacing rate for all the servers.
 	void set_pacing_rate(int stream_index, uint32_t pacing_rate);
@@ -52,9 +52,12 @@ public:
 	// Changes the given stream's amount of forced prebuffering on all the servers.
 	void set_prebuffering_bytes(int stream_index, size_t new_amount);
 
-	// Changes the given stream's encoding type on all the servers.
+	// Changes the given stream's output encoding type on all the servers.
 	void set_encoding(int stream_index, Stream::Encoding encoding);
 
+	// Changes the given stream's input encoding type on all the servers.
+	void set_src_encoding(int stream_index, Stream::Encoding encoding);
+
 	// Adds the given gen204 endpoint to all the servers.
 	void add_gen204(const std::string &url, const std::string &allow_origin);
 
