@@ -15,16 +15,11 @@
 class StreamProto;
 struct Client;
 
-enum StreamStartSuitability {
-	NOT_SUITABLE_FOR_STREAM_START,
-	SUITABLE_FOR_STREAM_START,
-};
-
 struct Stream {
 	// Must be in sync with StreamConfig::Encoding.
 	enum Encoding { STREAM_ENCODING_RAW = 0, STREAM_ENCODING_METACUBE };
 
-	Stream(const std::string &stream_id, size_t backlog_size, size_t prebuffering_bytes, Encoding encoding);
+	Stream(const std::string &stream_id, size_t backlog_size, size_t prebuffering_bytes, Encoding encoding, Encoding src_encoding);
 	~Stream();
 
 	// Serialization/deserialization.
@@ -51,6 +46,9 @@ struct Stream {
 	// be Metacube, for reflecting to another Cubemap instance).
 	Encoding encoding;
 
+	// What encoding we expect the incoming data to be in (usually Metacube).
+	Encoding src_encoding;
+
 	// The stream data itself, stored in a circular buffer.
 	//
 	// We store our data in a file, so that we can send the data to the
@@ -96,7 +94,7 @@ struct Stream {
 	// The data pointers in the iovec are owned by us.
 	struct DataElement {
 		iovec data;
-		StreamStartSuitability suitable_for_stream_start;
+		uint16_t metacube_flags;
 	};
 	std::vector<DataElement> queued_data;
 
@@ -107,7 +105,7 @@ struct Stream {
 
 	// Add more data to <queued_data>, adding Metacube headers if needed.
 	// Does not take ownership of <data>.
-	void add_data_deferred(const char *data, size_t bytes, StreamStartSuitability suitable_for_stream_start);
+	void add_data_deferred(const char *data, size_t bytes, uint16_t metacube_flags);
 
 	// Add queued data to the stream, if any.
 	// You should hold the owning Server's <mutex>.
