@@ -35,7 +35,7 @@ private:
 
 	// The URL and its parsed components.
 	std::string url;
-	std::string host, port, path;
+	std::string user, host, port, path;
 
 	// The HTTP header we're sending to clients.
 	std::string http_header;
